import { NextRequest } from "next/server";
import { createRequest, RequestMethod } from "node-mocks-http";

import mockConfig from "./mockConfig";

export async function createMockRequest({
    method = "POST",
    content_type = "text/plain",
    searchParams = new URLSearchParams(),
    body = undefined,
}: {
    method?: RequestMethod;
    content_type?: string;
    searchParams?: URLSearchParams;
    body?: unknown;
} = {}) {
    const req = await createRequest({
        method: method as RequestMethod,
        headers: {
            "Content-Type": content_type,
        },
        body: body ? toReadableStream(JSON.stringify(body)) : undefined,
    });

    // Add .json() method to the request
    req.json = async () => {
        return body;
    };

    // Add .nextUrl property to the request
    req.nextUrl = {
        searchParams,
    };

    // Create headers
    mockConfig.headers = new Headers(req.headers as HeadersInit);
    return {
        ...req,
        headers: mockConfig.headers,
        nextUrl: {
            searchParams,
        },
    } as unknown as NextRequest;
}

export function toReadableStream(value) {
    return new ReadableStream({
        start(controller) {
            controller.enqueue(value);
            controller.close();
        },
    });
}
