"use server";

import { errorToMessage } from "lib/actions/errorToMessage";
import { getSessionAction } from "lib/session";

import service from "@snip/database";

import { validatePW } from "components/form/validation";

type valid = -1 | 0 | 1;

export interface ChangePasswordState {
    message?: string;
    passwordChanged?: boolean;
    currentPassword: valid;
    newPassword: valid;
    confirmNewPassword: valid;
}

async function _changePwAction(
    prevState: ChangePasswordState,
    formData: FormData,
): Promise<ChangePasswordState> {
    const session = await getSessionAction();

    if (!session || !session.user) {
        throw Error("Not logged in!");
    }

    const user = await service.user.getById(session.user.id);
    if (!user) {
        throw Error("User not found!");
    }

    //Check if old pw is valid
    const old_pw = formData.get("currentPassword") as string;
    if (!old_pw) {
        prevState.currentPassword = 0;
        return { ...prevState };
    }
    const old_pw_valid = await service.user.login(user.email, old_pw);
    if (!old_pw_valid) {
        prevState.currentPassword = 0;
        return { ...prevState };
    }

    //Check if new pw is valid
    const new_pw = formData.get("newPassword") as string;
    if (!new_pw) {
        prevState.newPassword = 0;
        return { ...prevState };
    }

    let valid = false;
    try {
        valid = validatePW(new_pw);
    } catch (_e) {
        valid = false;
    }

    if (!valid) {
        prevState.newPassword = 0;
        return { ...prevState };
    }

    const success = await service.user.updatePassword(
        user.email,
        formData.get("newPassword") as string,
    );

    return { passwordChanged: success, ...prevState };
}

export const changePasswordAction = errorToMessage(_changePwAction);
