"use client";
import { useRedirect } from "lib/hooks/useRedirect";
import Link from "next/link";
import { Button } from "react-bootstrap";
import { useFormState } from "react-dom";

import { UserData } from "@snip/database/types";

import { loginSubmitAction } from "./action";

import { Form, FormHeader } from "components/form";
import { FormError } from "components/form/alert";
import { SecondaryButton, SubmitButton } from "components/form/buttons";
import { EmailInput, PasswordInput } from "components/form/inputs/text";

export default function Login() {
    return (
        <div className="d-flex flex-grow-1 justify-content-center align-items-center">
            <LoginForm />
        </div>
    );
}

function LoginForm() {
    const initialState: {
        message?: string;
        userData?: UserData;
    } = {
        userData: undefined,
        message: undefined,
    };

    const [state, formAction] = useFormState<{
        message: string;
        userData: UserData;
        // @ts-expect-error form
    }>(loginSubmitAction, initialState);
    // Redirect to default page if logged in
    const pending_direct = useRedirect("/", state.userData !== undefined);

    return (
        <Form action={formAction}>
            <FormHeader>Please login to your account</FormHeader>
            <EmailInput />
            <PasswordInput />
            <FormError>{state.message}</FormError>

            <div className="d-flex align-items-center text-center mb-3 flex-column">
                <SubmitButton style={{ width: "90%" }} pending={pending_direct}>
                    Log in
                </SubmitButton>
                <Link href="/sso" passHref legacyBehavior prefetch>
                    <Button
                        variant="outline-dark"
                        style={{ width: "90%", marginBottom: "0.5rem" }}
                    >
                        Log in with SSO
                    </Button>
                </Link>
                <Link className="text-muted" href="/forgotPassword">
                    Forgot password?
                </Link>
            </div>

            <div className="d-flex align-items-center justify-content-center">
                <p className="mb-0 me-2">Don&apos;t have an account?</p>
                <Link href="/signup">
                    <SecondaryButton>Create new</SecondaryButton>
                </Link>
            </div>
        </Form>
    );
}
