import jwt from "jsonwebtoken";

import { config } from "@snip/config/server";
import { UserData } from "@snip/database/types";
export interface ChangeEmailToken {
    time: number;
    from_email: string;
    to_email: string;
    user_id: number;
    type: "change_email";
}

export async function verifyChangeEmailToken(
    user: UserData,
    token: string,
): Promise<ChangeEmailToken> {
    // try to unseal token
    let tokenData: ChangeEmailToken;
    try {
        tokenData = (await jwt.verify(
            token,
            config.EMAIL.TOKEN_SECRET + user.email,
        )) as ChangeEmailToken;
    } catch (error) {
        throw new Error("Invalid token");
    }

    if (!tokenData.from_email || tokenData.from_email !== user.email) {
        throw new Error("Invalid token");
    }

    if (!tokenData.to_email || tokenData.type !== "change_email") {
        throw new Error("Invalid token");
    }

    return tokenData;
}

export async function signChangeEmailToken(
    user: UserData,
    new_email: string,
): Promise<string> {
    const data = {
        time: Date.now(),
        from_email: user.email,
        to_email: new_email,
        user_id: user.id,
        type: "change_email",
    };

    return await jwt.sign(data, config.EMAIL.TOKEN_SECRET + user.email, {
        expiresIn: "1h",
    });
}
