import jwt from "jsonwebtoken";

import { config } from "@snip/config/server";
import { CollaborationInviteData } from "@snip/database/types";

export interface CollaborationRequestToken {
    time: number;
    invite_id: number;
    type: "collaboration_request";
}

export async function verifyCollaborationRequestToken(
    user_id: number,
    token: string,
): Promise<CollaborationRequestToken> {
    // try to unseal token
    let tokenData: CollaborationRequestToken;
    try {
        tokenData = (await jwt.verify(
            token,
            config.EMAIL.TOKEN_SECRET + user_id,
        )) as CollaborationRequestToken;
    } catch (error) {
        throw new Error("Invalid token");
    }

    if (!tokenData.invite_id) {
        throw new Error("Invalid token");
    }
    if (tokenData.type !== "collaboration_request") {
        throw new Error("Invalid token");
    }

    return tokenData;
}

export async function signCollaborationRequestToken(
    invite: Required<CollaborationInviteData>,
): Promise<string> {
    const data: CollaborationRequestToken = {
        time: Date.now(),
        invite_id: invite.id,
        type: "collaboration_request",
    };

    return await jwt.sign(data, config.EMAIL.TOKEN_SECRET + invite.user_id, {
        expiresIn: invite.expire_at.getTime() - Date.now(),
    });
}
