import jwt from "jsonwebtoken";

import { config } from "@snip/config/server";
import { UserData } from "@snip/database/types";

interface VerifyEmailToken {
    time: number;
    email: string;
    user_id: number;
    type: "verify_email";
}

export async function verifyUserToken(
    user: UserData,
    token: string,
): Promise<boolean> {
    // try to unseal token
    let tokenData: VerifyEmailToken;
    try {
        tokenData = (await jwt.verify(
            token,
            config.EMAIL.TOKEN_SECRET + user.email,
        )) as VerifyEmailToken;
    } catch (error) {
        throw new Error("Invalid token");
    }
    if (!tokenData.email || tokenData.email !== user.email) {
        throw new Error("Invalid token");
    }

    return true;
}

export async function signUserToken(user: UserData): Promise<string> {
    const data = {
        time: Date.now(),
        email: user.email,
        user_id: user.id,
        type: "verify_email",
    };

    return await jwt.sign(data, config.EMAIL.TOKEN_SECRET + user.email, {
        expiresIn: "1h",
    });
}
